@@ -1,3 +1,21 @@
+#
+# Copyright (c) 2022 TUM Department of Electrical and Computer Engineering.
+#
+# This file is part of MLonMCU.
+# See https://github.com/tum-ei-eda/mlonmcu.git for further info.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 """Command line subcommand for the tune stage."""
 
 import copy
@@ -41,6 +59,8 @@ def handle(args, ctx=None):
     if ctx:
         handle_build(args, context)
     else:
+        args.features.append("autotune")  # TODO: enable autotuning automatically?
+        # args.features.append("autotuned")  # ?
         with mlonmcu.context.MlonMcuContext(path=args.home, lock=True) as context:
             handle_build(args, context)
             kickoff_runs(args, RunStage.TUNE, context)
