@@ -1,3 +1,21 @@
+#
+# Copyright (c) 2022 TUM Department of Electrical and Computer Engineering.
+#
+# This file is part of MLonMCU.
+# See https://github.com/tum-ei-eda/mlonmcu.git for further info.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 import io
 import csv
 import ast
@@ -28,7 +46,7 @@ class Metrics:
 
     def get_data(self, include_optional=False):
         return {
-            key: ast.literal_eval(value) if isinstance(value, str) else value
+            key: (ast.literal_eval(value) if len(value) > 0 else None) if isinstance(value, str) else value
             for key, value in self.data.items()
             if key not in self.optional_keys or include_optional
         }
